@@ -15,9 +15,19 @@
 # ==============================================================================
 """Chex: Testing made fun, in JAX!"""
 
+from chex._src.asserts import assert_axis_dimension
+from chex._src.asserts import assert_axis_dimension_gt
 from chex._src.asserts import assert_devices_available
+from chex._src.asserts import assert_equal
+from chex._src.asserts import assert_equal_rank
 from chex._src.asserts import assert_equal_shape
+from chex._src.asserts import assert_equal_shape_prefix
+from chex._src.asserts import assert_equal_shape_suffix
+from chex._src.asserts import assert_exactly_one_is_none
 from chex._src.asserts import assert_gpu_available
+from chex._src.asserts import assert_is_broadcastable
+from chex._src.asserts import assert_max_traces
+from chex._src.asserts import assert_not_both_none
 from chex._src.asserts import assert_numerical_grads
 from chex._src.asserts import assert_rank
 from chex._src.asserts import assert_scalar
@@ -28,9 +38,14 @@ from chex._src.asserts import assert_scalar_positive
 from chex._src.asserts import assert_shape
 from chex._src.asserts import assert_tpu_available
 from chex._src.asserts import assert_tree_all_close
+from chex._src.asserts import assert_tree_all_equal_comparator
+from chex._src.asserts import assert_tree_all_equal_shapes
+from chex._src.asserts import assert_tree_all_equal_structs
 from chex._src.asserts import assert_tree_all_finite
 from chex._src.asserts import assert_type
+from chex._src.asserts import if_args_not_none
 from chex._src.dataclass import dataclass
+from chex._src.dataclass import mappable_dataclass
 from chex._src.fake import fake_jit
 from chex._src.fake import fake_pmap
 from chex._src.fake import fake_pmap_and_jit
@@ -39,14 +54,20 @@ from chex._src.pytypes import Array
 from chex._src.pytypes import ArrayBatched
 from chex._src.pytypes import ArrayDevice
 from chex._src.pytypes import ArrayNumpy
+from chex._src.pytypes import ArraySharded
+from chex._src.pytypes import ArrayTree
 from chex._src.pytypes import Numeric
 from chex._src.pytypes import PRNGKey
 from chex._src.pytypes import Scalar
+from chex._src.pytypes import Shape
 from chex._src.variants import all_variants
+from chex._src.variants import ChexVariantType
+from chex._src.variants import params_product
 from chex._src.variants import TestCase
 from chex._src.variants import variants
 
-__version__ = "0.0.1"
+
+__version__ = "0.0.3"
 
 __all__ = (
     "all_variants",
@@ -54,9 +75,23 @@ __all__ = (
     "ArrayBatched",
     "ArrayDevice",
     "ArrayNumpy",
+    "ArraySharded",
+    "ArrayTree",
+    "assert_axis_dimension",
+    "assert_axis_dimension_gt",
     "assert_devices_available",
+    "assert_equal",
+    "assert_equal_rank",
     "assert_equal_shape",
+    "assert_equal_shape_prefix",
+    "assert_equal_shape_suffix",
+    "assert_exactly_one_is_none",
+    "assert_gpu_available",
+    "assert_is_broadcastable",
+    "assert_max_traces",
+    "assert_not_both_none",
     "assert_numerical_grads",
+    "assert_rank",
     "assert_scalar",
     "assert_scalar_in",
     "assert_scalar_negative",
@@ -65,17 +100,24 @@ __all__ = (
     "assert_shape",
     "assert_tpu_available",
     "assert_tree_all_close",
+    "assert_tree_all_equal_comparator",
+    "assert_tree_all_equal_shapes",
+    "assert_tree_all_equal_structs",
     "assert_tree_all_finite",
     "assert_type",
+    "ChexVariantType",
     "dataclass",
     "fake_jit",
     "fake_pmap",
     "fake_pmap_and_jit",
+    "if_args_not_none",
+    "mappable_dataclass",
     "Numeric",
+    "params_product",
     "PRNGKey",
-    "assert_rank",
     "Scalar",
     "set_n_cpu_devices",
+    "Shape",
     "TestCase",
     "variants",
 )
