@@ -23,6 +23,7 @@ See https://www.martinfowler.com/articles/mocksArentStubs.html
 
 import contextlib
 import functools
+import inspect
 import os
 import re
 from typing import Optional
@@ -81,25 +82,43 @@ def set_n_cpu_devices(n: Optional[int] = None):
       [f'--xla_force_host_platform_device_count={n}'] + xla_flags.split())
 
 
+def convert_to_varargs(sig, *args, **kwargs):
+  """Converts varargs+kwargs function arguments into varargs only."""
+  bound_args = sig.bind(*args, **kwargs)
+  return bound_args.args
+
+
 @functools.wraps(jax.jit)
 def _fake_jit(fn, *unused_args, **unused_kwargs):
   return fn
 
 
 @functools.wraps(jax.pmap)
-def _fake_pmap(fn, *unused_args, **unused_kwargs):
-  return jax.vmap(fn)
+def _fake_pmap(fn, axis_name=None, *, in_axes=0, static_broadcasted_argnums=(),
+               **unused_kwargs):
+  """Fake implementation of pmap using vmap."""
+  if static_broadcasted_argnums:
+    raise ValueError('fake pmap does not currently support non-empty '
+                     f'static_broadcasted_argnums={static_broadcasted_argnums}')
 
+  fn_signature = inspect.signature(fn)
+  vmapped_fn = jax.vmap(fn, in_axes=in_axes, axis_name=axis_name)
 
-def _zero(*unused_args, **unused_kwargs):
-  return 0
+  @functools.wraps(fn)
+  def wrapped_fn(*args, **kwargs):
+    # Convert kwargs to varargs
+    # This is a workaround for vmapped functions not working with kwargs
+    call_args = convert_to_varargs(fn_signature, *args, **kwargs)
+    output = vmapped_fn(*call_args)
+    return output
+
+  return wrapped_fn
 
 
 def _identity(x, *unused_args, **unused_kwargs):
   return x
 
 
-_fake_axis_index = functools.wraps(jax.lax.axis_index)(_zero)
 _fake_psum = functools.wraps(jax.lax.psum)(_identity)
 _fake_pmean = functools.wraps(jax.lax.pmean)(_identity)
 _fake_pmax = functools.wraps(jax.lax.pmax)(_identity)
@@ -112,6 +131,15 @@ def _fake_all_gather(x, *unused_args, **unused_kwargs):
   return jax.tree_map(add_leading_dim, x)
 
 
+class FakeContext(contextlib.ExitStack):
+
+  def start(self):
+    self.__enter__()
+
+  def stop(self):
+    self.__exit__(None, None, None)
+
+
 def fake_jit(enable_patching: bool = True):
   """Context manager for patching jax.jit with the identity function.
 
@@ -140,13 +168,13 @@ def fake_jit(enable_patching: bool = True):
   Returns:
     Context where jax.jit is patched with the identity function
   """
-  stack = contextlib.ExitStack()
+  stack = FakeContext()
   if enable_patching:
     stack.enter_context(mock.patch('jax.jit', _fake_jit))
   return stack
 
 
-def fake_pmap(enable_patching: bool = False):
+def fake_pmap(enable_patching: bool = True):
   """Context manager for patching jax.pmap with jax.vmap.
 
   This is intended to be used as a debugging tool to programmatically replace
@@ -175,10 +203,9 @@ def fake_pmap(enable_patching: bool = False):
     Context where jax.pmap is patched with jax.vmap
   """
   # Improve implementation to automatically track JAX collectives development.
-  stack = contextlib.ExitStack()
+  stack = FakeContext()
   if enable_patching:
     stack.enter_context(mock.patch('jax.pmap', _fake_pmap))
-    stack.enter_context(mock.patch('jax.lax.axis_index', _fake_axis_index))
     stack.enter_context(mock.patch('jax.lax.psum', _fake_psum))
     stack.enter_context(mock.patch('jax.lax.pmean', _fake_pmean))
     stack.enter_context(mock.patch('jax.lax.pmax', _fake_pmax))
@@ -194,6 +221,10 @@ def fake_pmap_and_jit(enable_pmap_patching: bool = True,
   This is a convenience function, equivalent to nested `chex.fake_pmap` and
   `chex.fake_jit` contexts.
 
+  Note that calling (the true implementation of) jax.pmap will compile the
+  function, so faking jax.jit in this case will not stop the function from
+  being compiled.
+
   Args:
     enable_pmap_patching: Whether to patch jax.pmap
     enable_jit_patching: Whether to patch jax.jit
@@ -202,7 +233,7 @@ def fake_pmap_and_jit(enable_pmap_patching: bool = True,
     Context where jax.pmap and jax.jit are patched with jax.vmap and the
     identity function
   """
-  stack = contextlib.ExitStack()
+  stack = FakeContext()
   stack.enter_context(fake_pmap(enable_pmap_patching))
   stack.enter_context(fake_jit(enable_jit_patching))
   return stack
