@@ -15,20 +15,23 @@
 # ==============================================================================
 """Pytypes for arrays and scalars."""
 
-from typing import Any, Iterable, Mapping, Union
+from typing import Any, Iterable, Mapping, Tuple, Union
 import jax
 import jax.numpy as jnp
 import numpy as np
 
 Array = jnp.ndarray
 ArrayBatched = jax.interpreters.batching.BatchTracer
-ArrayDevice = jax.interpreters.xla.DeviceArray
+# Use this type for type annotation. For instance checking,  use
+# `isinstance(x, jax.DeviceArray`.
+ArrayDevice = jax.interpreters.xla._DeviceArray  # pylint: disable=protected-access
 ArrayNumpy = np.ndarray
 ArraySharded = jax.interpreters.pxla.ShardedDeviceArray
 
 Scalar = Union[float, int]
 Numeric = Union[Array, Scalar]
 PRNGKey = Array
+Shape = Tuple[int, ...]
 
 # As of 06/2020 pytype doesn't support recursive types (see b/109648354)
 # pytype: disable=not-supported-yet
