@@ -15,6 +15,7 @@
 # ==============================================================================
 """Chex variants utilities."""
 
+import enum
 import functools
 import inspect
 import itertools
@@ -29,7 +30,6 @@ from jax import tree_util
 import jax.numpy as jnp
 import toolz
 
-
 FLAGS = flags.FLAGS
 flags.DEFINE_bool(
     "chex_skip_pmap_variant_if_single_device", True,
@@ -49,12 +49,39 @@ flags.DEFINE_bool(
 # instead of `variants.TestCase` or `parameterized.TestCase`. If a base class
 # doesn't support this feature variant test fails with a corresponding error.
 class TestCase(parameterized.TestCase):
-  pass
 
+  def variant(self, *args, **kwargs):
+    """To pass pytype checks."""
+    raise NotImplementedError("self.variant is not implemented: "
+                              "forgot to wrap a test in @chex.variants?")
+
+
+class ChexVariantType(enum.Enum):
+  WITH_JIT = 1
+  WITHOUT_JIT = 2
+  WITH_DEVICE = 3
+  WITHOUT_DEVICE = 4
+  WITH_PMAP = 5
 
 tree_map = tree_util.tree_map
 
 
+def params_product(*params_lists, named=False):
+  """Generate a cartesian product of params_lists."""
+
+  def generate():
+    for combination in itertools.product(*params_lists):
+      if named:
+        name = "_".join(t[0] for t in combination)
+        args_tuples = (t[1:] for t in combination)
+        args = sum(args_tuples, ())
+        yield (name, *args)
+      else:
+        yield sum(combination, ())
+
+  return list(generate())
+
+
 def count_num_calls(fn):
   """Counts number of function calls."""
   num_calls = 0
@@ -125,13 +152,13 @@ class VariantsTestCaseGenerator:
   def _inner_iter(self, test_method):
     """Generate chex variants for a single test."""
 
-    def make_test(variant):
+    def make_test(variant: ChexVariantType):
 
       @functools.wraps(test_method)
       def test(self, *args, **kwargs):
         # Skip pmap variant if only one device is available.
 
-        if (variant == "with_pmap" and
+        if (variant is ChexVariantType.WITH_PMAP and
             FLAGS.chex_skip_pmap_variant_if_single_device and
             jax.device_count() < 2):
           self.skipTest(f"Only 1 device is available ({jax.devices()}).")
@@ -154,6 +181,7 @@ class VariantsTestCaseGenerator:
 
         # Set up the variant.
         self.variant, num_calls = count_num_calls(_variant_decorators[variant])
+        self.variant.type = variant
         res = test_method(self, *args, **kwargs)
         if num_calls() == 0:
           raise RuntimeError(
@@ -178,6 +206,10 @@ class VariantsTestCaseGenerator:
 def _variants_fn(test_object, **which_variants):
   """Implements `variants` and `all_variants`."""
 
+  # Convert keys to enum entries.
+  which_variants = {
+      ChexVariantType[name.upper()]: var for name, var in which_variants.items()
+  }
   if isinstance(test_object, VariantsTestCaseGenerator):
     # Merge variants for nested wrappers.
     test_object.add_variants(which_variants)
@@ -305,11 +337,7 @@ def _with_jit(fn,
               **unused_kwargs):
   """Variant that applies `jax.jit` to fn."""
 
-  @functools.wraps(fn)
-  def wrapper(*args, **kwargs):
-    return jax.jit(fn, static_argnums, device, backend)(*args, **kwargs)
-
-  return wrapper
+  return jax.jit(fn, static_argnums, device, backend)
 
 
 @toolz.curry
@@ -326,11 +354,13 @@ def _without_jit(fn, **unused_kwargs):
 
 @toolz.curry
 @check_variant_arguments
-def _with_device(fn, ignore_argnums=(), **unused_kwargs):
+def _with_device(fn, ignore_argnums=(), static_argnums=(), **unused_kwargs):
   """Variant that applies `jax.device_put` to the args of fn."""
 
   if isinstance(ignore_argnums, int):
     ignore_argnums = (ignore_argnums,)
+  if isinstance(static_argnums, int):
+    static_argnums = (static_argnums,)
 
   @functools.wraps(fn)
   def wrapper(*args, **kwargs):
@@ -342,8 +372,8 @@ def _with_device(fn, ignore_argnums=(), **unused_kwargs):
         return x
 
     device_args = [
-        arg if idx in ignore_argnums else tree_map(put, arg)
-        for idx, arg in enumerate(args)
+        arg if (idx in ignore_argnums or idx in static_argnums) else tree_map(
+            put, arg) for idx, arg in enumerate(args)
     ]
     device_kwargs = tree_map(put, kwargs)
     return fn(*device_args, **device_kwargs)
@@ -426,7 +456,15 @@ def _with_pmap(fn,
   if isinstance(static_argnums, int):
     static_argnums = (static_argnums,)
 
-  @functools.wraps(fn)
+  pmap_kwargs = dict(
+      axis_name=axis_name,
+      devices=devices,
+      in_axes=in_axes,
+      static_broadcasted_argnums=static_argnums,
+      backend=backend)
+  pmapped_fn = jax.pmap(fn, **pmap_kwargs)
+
+  @functools.wraps(pmapped_fn)
   def wrapper(*args: pytypes.ArrayTree, **kwargs: pytypes.ArrayTree):
     if kwargs and (in_axes != 0 or static_argnums):
       raise ValueError("Do not use kwargs with `in_axes` or `static_argnums` "
@@ -463,27 +501,32 @@ def _with_pmap(fn,
               f"got: {shapes} (expected the first dim to be {n_devices_}). "
               "Consider setting `broadcast_args_to_devices=True`.")
 
-    res = jax.pmap(
-        fn,
+    new_kwargs = dict(
         axis_name=axis_name,
         devices=devices_,
         in_axes=in_axes,
         static_broadcasted_argnums=static_argnums,
-        backend=backend)(*args, **kwargs)
+        backend=backend)
 
+    # Re-compile fn if kwargs changed.
+    nonlocal pmap_kwargs
+    nonlocal pmapped_fn
+    if new_kwargs != pmap_kwargs:
+      pmap_kwargs = new_kwargs
+      pmapped_fn = jax.pmap(fn, **pmap_kwargs)
+
+    res = pmapped_fn(*args, **kwargs)
     return reduce_fn(res)
 
   return wrapper
 
-
-_variant_decorators = {
-    "with_jit": _with_jit,
-    "without_jit": _without_jit,
-    "with_device": _with_device,
-    "without_device": _without_device,
-    "with_pmap": _with_pmap,
-}
-
+_variant_decorators = dict({
+    ChexVariantType.WITH_JIT: _with_jit,
+    ChexVariantType.WITHOUT_JIT: _without_jit,
+    ChexVariantType.WITH_DEVICE: _with_device,
+    ChexVariantType.WITHOUT_DEVICE: _without_device,
+    ChexVariantType.WITH_PMAP: _with_pmap,
+})
 
 # Collect valid argument names from all variant decorators.
 _valid_kwargs_keys = set()
