@@ -18,6 +18,7 @@
 To run tests in multi-cpu regime, one need to set the flag `--n_cpu_devices=N`.
 """
 
+import inspect
 import itertools
 import unittest
 
@@ -198,6 +199,49 @@ def _variant_default_tests_generator(fn, is_jit_context, which_variants,
   return itertools.chain(*all_tests)
 
 
+class ParamsProductTest(absltest.TestCase):
+
+  def test_product(self):
+    l1 = (
+        ('x1', 1, 10),
+        ('x2', 2, 20),
+    )
+
+    l2 = (
+        ('y1', 3),
+        ('y2', 4),
+    )
+
+    l3 = (
+        ('z1', 5, 50),
+        ('z2', 6, 60),
+    )
+
+    l4 = (('aux', 'AUX'),)
+
+    expected = [('x1', 1, 10, 'y1', 3, 'z1', 5, 50, 'aux', 'AUX'),
+                ('x1', 1, 10, 'y1', 3, 'z2', 6, 60, 'aux', 'AUX'),
+                ('x1', 1, 10, 'y2', 4, 'z1', 5, 50, 'aux', 'AUX'),
+                ('x1', 1, 10, 'y2', 4, 'z2', 6, 60, 'aux', 'AUX'),
+                ('x2', 2, 20, 'y1', 3, 'z1', 5, 50, 'aux', 'AUX'),
+                ('x2', 2, 20, 'y1', 3, 'z2', 6, 60, 'aux', 'AUX'),
+                ('x2', 2, 20, 'y2', 4, 'z1', 5, 50, 'aux', 'AUX'),
+                ('x2', 2, 20, 'y2', 4, 'z2', 6, 60, 'aux', 'AUX')]
+    product = list(variants.params_product(l1, l2, l3, l4, named=False))
+    self.assertEqual(product, expected)
+
+    named_expected = [('x1_y1_z1_aux', 1, 10, 3, 5, 50, 'AUX'),
+                      ('x1_y1_z2_aux', 1, 10, 3, 6, 60, 'AUX'),
+                      ('x1_y2_z1_aux', 1, 10, 4, 5, 50, 'AUX'),
+                      ('x1_y2_z2_aux', 1, 10, 4, 6, 60, 'AUX'),
+                      ('x2_y1_z1_aux', 2, 20, 3, 5, 50, 'AUX'),
+                      ('x2_y1_z2_aux', 2, 20, 3, 6, 60, 'AUX'),
+                      ('x2_y2_z1_aux', 2, 20, 4, 5, 50, 'AUX'),
+                      ('x2_y2_z2_aux', 2, 20, 4, 6, 60, 'AUX')]
+    named_product = list(variants.params_product(l1, l2, l3, l4, named=True))
+    self.assertEqual(named_product, named_expected)
+
+
 class FailedTestsTest(variants.TestCase):
   # Inner class prevents FailedTest being run by `absltest.main()`.
 
@@ -214,7 +258,7 @@ class FailedTestsTest(variants.TestCase):
 
   def setUp(self):
     super().setUp()
-    self.chex_info = '(chex variant == `without_jit`)'
+    self.chex_info = f'variant == `{variants.ChexVariantType.WITHOUT_JIT}`'
     self.res = unittest.TestResult()
     ts = unittest.makeSuite(self.FailedTest)  # pytype: disable=module-attr
     ts.run(self.res)
@@ -247,8 +291,8 @@ class OneFailedVariantTest(variants.TestCase):
       fails_for_without_device_variant(42)
 
   def test_useful_failure(self):
-    expected_info = '(chex variant == `without_device`)'
-    unexpected_info = '(chex variant == `with_device`)'
+    expected_info = f'variant == `{variants.ChexVariantType.WITHOUT_DEVICE}`'
+    unexpected_info = f'variant == `{variants.ChexVariantType.WITH_DEVICE}`'
 
     res = unittest.TestResult()
     ts = unittest.makeSuite(self.MaybeFailedTest)  # pytype: disable=module-attr
@@ -408,6 +452,34 @@ class UnknownVariantArgumentsTest(absltest.TestCase):
       self.assertRegex(msg, 'Unknown arguments in .+some_unknown_arg')
 
 
+class VariantTypesTest(absltest.TestCase):
+  # Inner class prevents InnerTest being run by `absltest.main()`.
+
+  class InnerTest(variants.TestCase):
+    var_types = set()
+
+    @variants.all_variants()
+    def test_var_type(self):
+      self.variant(lambda: None)
+      self.var_types.add(self.variant.type)
+
+  def test_var_type_fetch(self):
+    ts = unittest.makeSuite(self.InnerTest)  # pytype: disable=module-attr
+    ts.run(unittest.TestResult())
+    expected_types = set(variants.ChexVariantType)
+    if jax.device_count() == 1:
+      expected_types.remove(variants.ChexVariantType.WITH_PMAP)
+    self.assertSetEqual(self.InnerTest.var_types, expected_types)
+
+  def test_consistency(self):
+    self.assertLen(variants._variant_decorators, len(variants.ChexVariantType))
+
+    for arg in inspect.getfullargspec(variants.variants).args:
+      if arg == 'test_method':
+        continue
+      self.assertTrue(hasattr(variants.ChexVariantType, arg.upper()))
+
+
 class CountVariantsTest(absltest.TestCase):
   # Inner class prevents InnerTest being run by `absltest.main()`.
 
